import gnu.crypto.cipher.*;
import gnu.crypto.mode.*;
import gnu.crypto.pad.*;
import java.util.*;

public class AESCrypt {
	private int debug;
	
	//Constructor will set debug variable from user
	public AESCrypt(int db) {debug=db;}

	//Encrypts String message using AES and key String key of int length
	public byte[] encrypt (String key, String message, int length) {
		
		//Create byte arrays of the key, the message, and the ciphertext
		//byte arrays are returned from the encryption algorithm and must
		//be converted to strings before return
		byte key_bytes[]=key.getBytes();		
		byte pt[] = message.getBytes();
		byte ct[] = new byte[pt.length];
		
		//Initialize the cipher object to use AES and a block size
		//dictated by the user
		IBlockCipher cipher = CipherFactory.getInstance("AES");
		Map attrib = new HashMap();
		attrib.put(IBlockCipher.CIPHER_BLOCK_SIZE, new Integer(length));
		attrib.put(IBlockCipher.KEY_MATERIAL, key_bytes);
		try {
			cipher.init(attrib);
		}
		catch(Exception ex) {
			System.err.println("init() : " + ex);
			return("oops".getBytes());
		}

		//Set the text blocksize
		int bs = cipher.currentBlockSize();

		if( ! cipher.selfTest() ) {
			System.err.println("Self test fails....");
		}

		if(debug==1)System.out.println("Key Size:   " + cipher.currentBlockSize());
		if(debug==1)System.out.println("Block Size: " + bs);
		if(debug==1)System.out.println("PlainText: |" + new String(pt) + "|");

		//Encrypt a block of text for as long as there are whole blocks left
		//to encrypt. Will not encrypt partial blocks, be sure to send a
		//number of characters stretched to the nearest block size
		for (int i = 0; i + bs <= pt.length; i += bs) cipher.encryptBlock(pt, i, ct, i);

		if(debug==1)System.out.println("Encrypted: |" + new String(ct) + "|");
		
		//Return the ciphertext as a string
		return(ct);
		}
	
	//Decrypts String message using AES and String key of int length
	public String decrypt (String key, byte[] message, int length) {
		
		//Set up the byte arrays for key, ciphertext, and decrypted plaintext
		byte key_bytes[]=key.getBytes();		//turns the key into a byte array
		byte ct[] = message;                            //ciphertext becomes a byte array of the cipher passed
		byte cpt[] = new byte[ct.length];		//plaintext becomes a byte array of length of ciphertext
		
		//Initialize the cipher. Set blocksize to the length of the key
		IBlockCipher cipher = CipherFactory.getInstance("AES");
		Map attrib = new HashMap();
		attrib.put(IBlockCipher.CIPHER_BLOCK_SIZE, new Integer(length));
		attrib.put(IBlockCipher.KEY_MATERIAL, key_bytes);
		try {
			cipher.init(attrib);
		}
		catch(Exception ex) {
			System.err.println("init() : " + ex);
			return("oops");
		}
	
		//Set the blocksize
		int bs = cipher.currentBlockSize();

		//Decrypt text in blocks of size blocksize and append them together
		//Make sure you send text an exact block size in length.
		/*TODO: check message length before encrypting or decrypting?
		        Move the check code out of the client and server to make it more
		        uniform. We have all of the needed variables here anyway.*/
		for (int i = 0; i + bs <= cpt.length; i += bs) cipher.decryptBlock(ct, i, cpt, i);

		if(debug==1)System.out.println("Decrypted: |" + new String(cpt) + "|");

		//Return a string containing the decrypted plaintext.
		return(new String(cpt));
	}

}
