import java.io.*;
import java.lang.ClassNotFoundException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.math.BigInteger;
import java.util.Scanner;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import java.awt.*;
import java.lang.Runnable;
import java.lang.Thread;
  
public class CAEMURA {

	//GUI builder object
	public static CAEMURA_GUI mBorrowedResource;
	 
	//TURN ON DEBUGGING HERE FOR APPLICATION TESTING
	static int debug=0;
	
	//What is the IP of the host?
	static String host_ip;	
		
	//Initialize the RNG
	static RandomPrime prime = new RandomPrime();
			
	//Initialize strings used for messages
	static String message;     //messages in the DH exchange
	static byte recvmsg[];     //Encrypted byte[] received from remote user
	static String recvmsg2;    //Decrypted String made from the encrypted byte[]
	static String sendmsg="";  //Unencrypted string to send remote user
	static byte sendmsg2[];    //Encrypted byte[] send to remote user
			
	//Integer will store key length for comparisons
	static int length=0;
		
	//Initialize encryption/decryption object.
	//Pass it debug so it will turn on debug mode if necessary
	static AESCrypt A = new AESCrypt(debug);
			
	//Initialize various components for key generation
	static USER userA;                 //User object will store key parts
	static BigInteger user_A_private;  //Private part of local user
	static BigInteger YB;              //Public part of remote user
	static BigInteger UserAKey=new BigInteger("0");  //final key
			
	//Initialize various input and output streams
	static BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	static ObjectInputStream ois;
	static ObjectOutputStream oos;

	//Triggered by ActionListener on "Connect" button.
	//Prompts user for ip of server (defaults to localhost)
	public static void getIP(){
		host_ip = JOptionPane.showInputDialog("What IP are you connecting to (leave blank for localhost)?");
		if(host_ip=="") host_ip="127.0.0.1";	
	}

	//Triggered by the "connect" button after getIP()
	//Opens socket with server and starts listening for incoming messages
	public static void connect(){
		try{
			//Open a socket, and I/O streams with the user's host
			Socket socket = new Socket(host_ip, 7777);
			mBorrowedResource.mainConsole.append("Unsecure connection established with server. \n\nPerforming Diffie-Hellman Key Exchange...\n\n");
			oos = new ObjectOutputStream(socket.getOutputStream());
			ois = new ObjectInputStream(socket.getInputStream());
			generateKey();
			mBorrowedResource.setLocked();

			//Thread created to listen for messages coming in from the server
			Thread listeningProcedure = new Thread(
				new Runnable() {
					public void run() {
						while(true) {
						CAEMURA.recvMessage();
		      				}
		  			}
		  		}
			);
		
			listeningProcedure.start();
		}
		catch (UnknownHostException e) {
			e.printStackTrace();
	        }
		catch (IOException e) {
			e.printStackTrace();}
	}
	
	//Triggered by connect()
	//Generates key using Diffie-Hellman
	public static void generateKey(){
		try{
			//Make sure key is 128bit
			while(length!=16){
			
				//Generate Q and A key parts
				BigInteger q = prime.getNum(80);			
				BigInteger a = prime.getNum(3);					
			
				//Create User A (local user)
				user_A_private = prime.getNum(50);
				userA = new USER(q, a, user_A_private);	
		
				// Send Q and A to remote user
				oos.writeObject(q.toString());
				oos.writeObject(a.toString());
			
				//Receive the remote user's public part (YB)
				message = (String) ois.readObject();  // here, we get an object
				if(debug==1)mBorrowedResource.mainConsole.append("YB: " + message);
				YB = new BigInteger(message);
				userA.OtherUserPublic(YB);
			
				//Send the remote user our public part
				oos.writeObject(userA.getPublic().toString());
			
				//Calculate the key
				UserAKey = userA.CalculateKey(YB);
				if(debug==1)mBorrowedResource.mainConsole.append("Key:"+UserAKey.toString());
			
				mBorrowedResource.mainConsole.append("Connection Secured!\n");
				mBorrowedResource.mainConsole.append("Entering chat mode. ");
				mBorrowedResource.mainConsole.append("All messages will be encrypted with Rijndael before transmission.\n\n\n");
			
				//Truncate the generated 192-bit key to 128-bit
				UserAKey=new BigInteger(UserAKey.toString().substring(0,16));
			
				//Set length variable for comparisons. Length of key
				byte[] KeyInBytes = UserAKey.toString().getBytes();
				length = KeyInBytes.length;

				if(length!=16) mBorrowedResource.mainConsole.append("Key length incorrect. Regenerating...");
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	//Triggered by the send button
	//Oddly enough, sends a message
	public static void sendMessage(){
		try{
			//Get message to send
			mBorrowedResource.mainConsole.append("Me: ");
			sendmsg=mBorrowedResource.inputMessage.getText();
			
			mBorrowedResource.mainConsole.append(sendmsg);
			//Stretch message to an exact number of blocks for encryption
			while(sendmsg.length()%length > 0) sendmsg+=" ";
				
			//Encrypt message
			sendmsg2 = A.encrypt(UserAKey.toString(), sendmsg, length);
			if(debug==1)mBorrowedResource.mainConsole.append(A.decrypt(UserAKey.toString(),sendmsg2,length));
			
			//Check if decryptable before sending it
			while(!A.decrypt(UserAKey.toString(),sendmsg2,length).substring(0,1).equals(sendmsg.substring(0,1))){
				mBorrowedResource.mainConsole.append("\nError in decryption. Please choose a different message.");
				sendmsg = mBorrowedResource.inputMessage.getText();
				
				while(sendmsg.length()%length > 0) sendmsg+=" ";
				sendmsg2 = A.encrypt(UserAKey.toString(), sendmsg, length);
			}
		
			mBorrowedResource.inputMessage.setText("");
		
			//Send the message
			oos.writeObject(sendmsg2);
				
			//Does the user want to quit?
			if(sendmsg.substring(0,5).equals("/quit")) disconnect();

			mBorrowedResource.mainConsole.append("\n");
		}
		catch (IOException e) {
			e.printStackTrace();
           	}	
	}
	
	//Used in the listening thread
	//If there is a message, print it.
	public static void recvMessage(){
		
		try{
			//Receive a message
			recvmsg = (byte[])ois.readObject();
			recvmsg2 = A.decrypt(UserAKey.toString(),recvmsg,length);
					
			//Did the server disconnect?
			if(recvmsg2.length() >= 5 && recvmsg2.substring(0,5).equals("/quit")) disconnect();
			mBorrowedResource.mainConsole.append("Them: ");		
			mBorrowedResource.mainConsole.append(recvmsg2+"\n");
		}
		catch (IOException e) {
			disconnect();
     		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	//Triggered by disconnect button
	//Closes the socket
	public static void disconnect(){
		
		try{		
			ois.close();
			oos.close();
			System.exit(0);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	//Main method
	//Starts objects and stuff
	public static void main(String[] args) {
		mBorrowedResource = new CAEMURA_GUI();
		mBorrowedResource.setVisible(true);
	}
}

class CAEMURA_GUI extends JFrame { 
	
	//Define GUI Objects
	public final JTextArea mainConsole = new JTextArea(""); 	 
	public final JButton connect = new JButton("Connect"); 
	public final JButton disconnect = new JButton("Disconnect"); 
	public final JButton sendMessage = new JButton("Send Message"); 
	public final JTextField inputMessage = new JTextField("");
	public final ImageIcon unlock = new ImageIcon("../img/unlock.png","Not Connected or Not Encrypted!");
	public final ImageIcon lock = new ImageIcon("../img/lock.png","Connected and Encrypted!");
	public final JLabel labelUnlock = new JLabel();
	public final JLabel labelLock = new JLabel();
	public JPanel panel;

	public void setLocked(){
		panel.add(labelLock);
		panel.remove(labelUnlock);
		panel.repaint();
	}

	//CAEMURA Constructor
	//Controls, objects, and actionlisteners
	CAEMURA_GUI()
	{
   
		panel = new JPanel();
		getContentPane().add(panel);
		panel.setLayout(null);
 
		setDefaultCloseOperation(javax.swing.
		WindowConstants.DISPOSE_ON_CLOSE);
		setTitle("CAEMURA");
		setSize(800, 670);	
	
		JLabel labelOne = new JLabel("Client Application for Encrypted Messaging Using Rijndael Algorithm\n(CAEMURA) v0.9 June 22, 2009\n\n");
		labelOne.setBounds(10,1,800,100);
	 
		labelUnlock.setIcon(unlock);
		labelUnlock.setBounds(730,570,39,57);

		labelLock.setIcon(lock);
		labelLock.setBounds(730,570,39,57);

		mainConsole.setBounds(10, 60, 780, 500);
		mainConsole.setEditable(false);

		//Put a ScrollPane over our textarea. <3 scrolling
		JScrollPane scrollPane = new JScrollPane(mainConsole);
		scrollPane.setBounds(10,60,780,500);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);	
		
	
		connect.setBounds(10, 10, 150, 30);

		//ActionListener for connect button		
		connect.addActionListener(new ActionListener() {
            		public void actionPerformed(ActionEvent event) { 
				//Call Connect Function
				CAEMURA.getIP();
				CAEMURA.connect();

			}
		});
		
		disconnect.setBounds(640, 10, 150, 30);
		
		//ActionListener for disconnect button
		disconnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) { 
				//Call Disconnect Function
				CAEMURA.disconnect();
			}
        	});

		sendMessage.setBounds(10, 600, 150,30);

		//ActionListener for sendMessage button		
		sendMessage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) { 
				//Call Send Message Function
				CAEMURA.sendMessage();
			}
		});
	 
		inputMessage.setBounds(10, 570, 700, 30);
		
		//ActionListener for inputMessage textField (nothing right now)
		inputMessage.addActionListener(new ActionListener() {
        		public void actionPerformed(ActionEvent event) {
				CAEMURA.sendMessage();
		}});

		//Add everything to the panel
		panel.add(labelUnlock);		
		panel.add(labelOne);
		panel.add(sendMessage);	
		panel.add(inputMessage);	
		panel.add(disconnect);
		panel.add(connect);	
		panel.add(scrollPane);
		
	}	
}
