import java.math.BigInteger;
import java.util.Random;
import java.security.SecureRandom;


class USER {
	private BigInteger q;		// This is the q (the modulo)
	private BigInteger a;		// this is the a, which is < q
	private BigInteger X;		// our PRIVATE component of the key
	private BigInteger Y;		// our PUBLIC component of the key

	private BigInteger Y_otherUser;	// The other user's PUBLIC component of the key



	/**
	 * Pass to us the q,a and our PRIVATE component of our key (the X).
	 * We then calculate our PUBLIC component of our key (the Y)
	 */
	public USER(BigInteger q, BigInteger a, BigInteger X) {
		this.q = q;
		this.a = a;
		this.X = X;

		Y = a.modPow(X,q);	// having a,q, and X, we can calculate Y.
	}

	/**
	 * Returns our PUBLIC component of our key.
	 * The other user will need this to calculate the final secret key.
	 */
	public BigInteger getPublic() {
		return Y;
	}


	/**
	 * We need to know the other user's PUBLIC component of the key.
	 * Store it so we can use it to calculate our final secret key.
	 */
	public void OtherUserPublic(BigInteger Y_otherUser) {
		this.Y_otherUser = Y_otherUser;
	}

	/**
	 * Calculates and returns the SECRET KEY (in the slides this is the K);
	 */
	public BigInteger CalculateKey(BigInteger Y_other) {
		return Y_other.modPow(X,q);
	}
}

/**
 * Generates random prime numbers for use in creating the key.
 * An object is created in DH_TEST to generate random primes for the
 * q, Xa, and Xb values. Ensuring a new key each time the program is run.
 */
class RandomPrime {
	private int bits;
	SecureRandom random = new SecureRandom();
	
	public RandomPrime(){}
	
	/**
	 * Returns what is most probably a prime number of length ubits bytes
	 * returns it as a string so it's a BigInteger can be created out of
	 * it back in DH_TEST
	 */
	public BigInteger getNum(int ubits){
		bits=ubits;
		BigInteger number = BigInteger.probablePrime(bits, random);
		return(number);
	}
}