import java.net.Socket;
import java.math.BigInteger;
import java.io.*;
import java.lang.ClassNotFoundException;
import java.lang.Runnable;
import java.lang.Thread;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.*;
import java.awt.*;

//SAEMURA class
//Instantiates the server object and lets us listen for a connection 
public class SAEMURA {
	private static ServerSocket server;
    
	//Set port for connection here
	private static  int port = 7777;
 
	public SAEMURA() {
		try {
			server = new ServerSocket(port);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
 
	public static void main(String[] args) {
		SAEMURA server = new SAEMURA();
		server.handleConnection();
	}
 
	public static void handleConnection() {
		//Open socket, listen for a connection from remote user
		while (true) {
			try {
				Socket socket = server.accept();
				new ConnectionHandler(socket);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
 
//ConnectionHandler class
//Thread that executes when we get a connection
class ConnectionHandler implements Runnable {

	//Static object used to reference GUI controls
	public static SAEMURA_GUI mBorrowedResource;
	
	//TURN ON DEBUGGING HERE FOR APPLICATION TESTING
	static int debug=0;
			
	//Initialize RNG
        static RandomPrime prime = new RandomPrime();
			
	//Initizalize message strings
	static String message;		//used to pass key information to and from remote user
	static String recvmsg;		//unencrypted string of byte[] received from remote user
	static byte recvmsg2[];		//encrypted byte[] received from remote user
	static String sendmsg;		//unencrypted string to encrypt and send to remote user
	static byte sendmsg2[];		//encrypted byte[] sent to remote user
			
	//used to make sure key is of correct length
	//and regulate block size of messages
	static int length=0;
			
	//used for key generation (diffie-hellman)
	static BigInteger q;                 //first large prime number
	static BigInteger a;                 //second small prime number
	static BigInteger user_B_private;    //private part of local user
	static BigInteger YA;                //public part of remote user
	static BigInteger UserBKey=new BigInteger("0");  //final generated key
	static USER userB;                   //"USER" object for local user 
	static private Socket socket;
	
	//Initialize the I/O Streams
	static BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	static ObjectOutputStream oos;
	static ObjectInputStream ois;
	
	//Initialize the encryption/decryption object
	static AESCrypt B = new AESCrypt(debug); 

	
	//Constructor starts the thread on current socket
	public ConnectionHandler(Socket socket) {
		this.socket = socket;
		
		//Start the main thread once a connection is established
		Thread t = new Thread(this);
		t.start();
	}
       
	//Open streams on the socket. Begin key generation
	//Start listener thread
	public static void connect(){
		try { 
			oos = new ObjectOutputStream(socket.getOutputStream());
			ois = new ObjectInputStream(socket.getInputStream());
			generateKey();
			mBorrowedResource.setLocked();
		}
		catch (IOException e) {
			e.printStackTrace();}
            
		//Define listener thread
		Thread listeningProcedure = new Thread(
			new Runnable() {
				public void run() {
					while(true) {
						ConnectionHandler.recvMessage();
					}
				}
			}
		); 
		
		//Start listener thread
		listeningProcedure.start();   
	}
	 

	
	//Hooks up with generatekey in the client app to perform a DH exchange
	public static void generateKey(){
		try{
			//Make sure key is 128bits
			while(length!=16){
			
			//Receive Q from Remote User
			message = (String) ois.readObject();
			if(debug==1)mBorrowedResource.mainConsole.append("Q: " + message);
			q = new BigInteger(message);
			
			//Receive A from Remote User
			message = (String) ois.readObject();
			if(debug==1)mBorrowedResource.mainConsole.append("A: " + message);
			a = new BigInteger(message);
 
				//Create User object for local user
				//and generate our public and private key parts
				user_B_private = prime.getNum(50);
				userB = new USER(q, a, user_B_private);
			
				//Send Remote user our public part
				oos.writeObject(userB.getPublic().toString());
			
				//Receive Remote User's public part
				message = (String) ois.readObject();
				if(debug==1)mBorrowedResource.mainConsole.append("YA: " + message);
				YA = new BigInteger(message);
				userB.OtherUserPublic(YA);
			
				//Generate Key
				UserBKey=userB.CalculateKey(YA);
				if(debug==1)mBorrowedResource.mainConsole.append("Key: "+UserBKey.toString());
			
				//Truncate 192-bit key to 128-bits
				UserBKey=new BigInteger(UserBKey.toString().substring(0,16));
			
				//Get key length
				byte[] KeyInBytes = UserBKey.toString().getBytes();
				length = KeyInBytes.length;
			
				if(length!=16) mBorrowedResource.mainConsole.append("Key length incorrect. Regenerating...");
			}
		}catch (IOException e) {
            		e.printStackTrace();}
       		 catch (ClassNotFoundException e) {
            e.printStackTrace();
        	}
	}
	
	//Gets a message from local user and sends it to the client	
	public static void sendMessage(){
		try{
			//Get a message to send
				
			sendmsg=mBorrowedResource.inputMessage.getText();
			mBorrowedResource.mainConsole.append("Me: "+sendmsg+"\n");
			
			//Make message an exact number of blocks in length
			//so it can be encrypted
			while(sendmsg.length()%length > 0) sendmsg+=" ";
			
			//Encrypt message, save it sendmsg2
			sendmsg2 = B.encrypt(UserBKey.toString(), sendmsg, length);
			if(debug==1)mBorrowedResource.mainConsole.append("Encrypted Message: "+B.decrypt(UserBKey.toString(),sendmsg2,length));
			
			//Check if message is decryptable before sending
			while(!B.decrypt(UserBKey.toString(),sendmsg2,length).substring(0,2).equals(sendmsg.substring(0,2))){
					
				//While it isn't decryptable, get a new one from user
				mBorrowedResource.mainConsole.append("\nError in decryption. Please choose a different message.");
				sendmsg = mBorrowedResource.inputMessage.getText();
				
				//Make the message an exact number of blocks
				while(sendmsg.length()%length > 0) sendmsg+=" ";
				sendmsg2 = B.encrypt(UserBKey.toString(), sendmsg, length);
			}
				
			mBorrowedResource.inputMessage.setText("");
			//Send the encrypted message
			oos.writeObject(sendmsg2);
		}catch (IOException e) {
            		e.printStackTrace();}			
	
	}
	
	//Run by listener thread.
	//Receives, decrypts, and displays incoming messages	
	public static void recvMessage(){
		try{
			//Receive a message
			recvmsg2 = (byte[])ois.readObject();
			
			//Decrypt received message
			recvmsg = B.decrypt(UserBKey.toString(),recvmsg2,length);
			mBorrowedResource.mainConsole.append("Them: "+recvmsg+"\n");	

		}catch (IOException e) {
            		disconnect();
		}catch (ClassNotFoundException e) {
            		e.printStackTrace();
	        }		
	}
		
	public static void disconnect(){
		try{
			ois.close();
			oos.close();
			socket.close();
			System.exit(0);
		}
        	catch (IOException e) {
            		e.printStackTrace();} 
	}
	
	public void run() {
		mBorrowedResource = new SAEMURA_GUI();
		mBorrowedResource.setVisible(true);
        
	//Beep when a client connects
	System.out.print("\007");System.out.flush();

	//Initiate connection and DH
	connect();

	mBorrowedResource.mainConsole.append("Client connected on port 7777!");
	mBorrowedResource.mainConsole.append("\n\nPerforming Diffie-Hellman Key Exchange...");
			
	mBorrowedResource.mainConsole.append("\n\nConnection Secured!\nEntering chat mode. ");
	mBorrowedResource.mainConsole.append("All messages will be encrypted with Rijndael before transmission.\n\n\n");
	}
}

class SAEMURA_GUI extends JFrame { 
	
	//Define GUI Objects
	public final JTextArea mainConsole = new JTextArea(""); 	 
	public final JButton connect = new JButton("Connect"); 
	public final JButton disconnect = new JButton("Disconnect"); 
	public final JButton sendMessage = new JButton("Send Message"); 
	public final JTextField inputMessage = new JTextField("");
	public final ImageIcon unlock = new ImageIcon("../img/unlock.png","Not Connected or Not Encrypted!");
	public final ImageIcon lock = new ImageIcon("../img/lock.png","Connected and Encrypted!");
	public final JLabel labelUnlock = new JLabel();
	public final JLabel labelLock = new JLabel();
	public JPanel panel;
	
	public void setLocked(){
		panel.add(labelLock);
		panel.remove(labelUnlock);
		panel.repaint();
	}
	
	//SAEMURA Constructor
	SAEMURA_GUI()
	{

   
		panel = new JPanel();
		getContentPane().add(panel);
		panel.setLayout(null);

		setDefaultCloseOperation(javax.swing.
		WindowConstants.DISPOSE_ON_CLOSE);
		setTitle("SAEMURA");
		setSize(800, 670);	

		JLabel labelOne = new JLabel("Server Application for Encrypted Messaging Using Rijndael Algorithm\n(CAEMURA) v0.9 June 22, 2009\n\n");
		labelOne.setBounds(10,1,800,100);

		labelUnlock.setIcon(unlock);
		labelUnlock.setBounds(730,570,39,57);

		labelLock.setIcon(lock);
		labelLock.setBounds(730,570,39,57);
	
		JScrollPane scrollPane = new JScrollPane(mainConsole);
		scrollPane.setBounds(10,60,780,500);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
	
		mainConsole.setBounds(10, 60, 780, 500);
		mainConsole.setEditable(false);

		disconnect.setBounds(640, 10, 150, 30);
		disconnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) { 
				//Call Disconnect Function
				ConnectionHandler.disconnect();
			}
		});

		sendMessage.setBounds(10, 600, 150,30);
		sendMessage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) { 
				//Call Send Message Function
				ConnectionHandler.sendMessage();
			}
		});
	 
		inputMessage.setBounds(10, 570, 700, 30);
		inputMessage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				ConnectionHandler.sendMessage();
			}
		});

		panel.add(labelUnlock);
		panel.add(labelOne);
		panel.add(scrollPane);
		panel.add(sendMessage);	
		panel.add(inputMessage);	
		panel.add(disconnect);
	}
	
}
